/*  <lalVerbatim file="LALColoredNoiseCV">
Author: Sathyaprakash, B. S.
$Id: LALColoredNoise.c,v 1.2 2001/03/10 19:14:04 jolien Exp $
</lalVerbatim>  */

/* <lalLaTeX>
\subsection{Module \texttt{LALColoredNoise.c}}
This module colors a given white noise input into a colored noise
of power spectral density \texttt{psd}.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{LALColoredNoiseCP}
\index{\texttt{LALColoredNoise()}}

\subsubsection*{Description}
\subsubsection*{Algorithm}
\subsubsection*{Uses}
\begin{verbatim}
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{LALColoredNoiseCV}}
</lalLaTeX>  */

#include <lal/LALNoiseModels.h>
NRCSID (LALCOLOREDNOISEC, "$Id: LALColoredNoise.c,v 1.2 2001/03/10 19:14:04 jolien Exp $");

/*  <lalVerbatim file="LALColoredNoiseCP"> */
void 
LALColoredNoise (
   LALStatus   *status,
   REAL4Vector *noisy, 
   REAL8Vector  psd) 
{  /*  </lalVerbatim>  */

   INT4 i, j, n, nby2;
   REAL8 x, length;
   

   INITSTATUS (status, "LALColoredNoise", LALCOLOREDNOISEC);
   if (psd.length != noisy->length/2+1) {
     fprintf(stderr, "LALColoredNoise: Incompatible lengths\n");
     exit(1);
   }
   n = length = noisy->length;
   nby2 = n/2;
   noisy->data[0] = 0.;
   noisy->data[nby2] = 0.;

   for (i=1; i<nby2; i++) {
      j = n-i;
      x = sqrt(2. * psd.data[i+1] / length);
      noisy->data[i] *= x;
      noisy->data[j] *= x;
   }
   RETURN(status); 
}
